#include <u.h>
#include <libc.h>
#include <ip.h>
#include "../lib/asn1.h"
#include "../lib/snmp.h"
#include "../lib/snmp_debug.h"


void usage(char *cmd)
{
		print("Usage: %s [-D] [-W|-B|-S] -h <host> [-p port] [-v ver] [-r <retries>] [-c <rocommunity>] [-w <rwcommunity>] -o <oid>\n", cmd);
		print("Example: %s -D -h 192.168.1.1 -p snmp -v 1 -r 2 -c public -o 1.3.6.1.2.1.1.1.0\n", cmd);
		exits("Error in args");

}


int
main(int argc, char *argv[])
{
	char *cmd, *host, *port, *rocomm, *rwcomm, *val, *oids, *values;
	char **param = nil;
	int version, retries, timeout, n, m, op;
	snmp_session *sess;

	/* default values */
	timeout = 5000;
	version = retries = n = m = 0;
	port = oids = values = rocomm = rwcomm = host = nil;
	op = GetRequest;

	cmd = argv[0];

	ARGBEGIN {
		case 'D':
			setdebug();
			break;
		case 'W':
			op = GetNextRequest;
			break;
		case 'B':
			op = GetBulkRequest;
			if ( getfields(ARGF(), param, 2, 0, ", ") < 2 )
				usage(cmd);
			n = atoi(param[0]);
			m = atoi(param[1]);
			break;
		case 'S':
			op = SetRequest;
			break;
		case 'h':
			host=ARGF();
			break;
		case 'p':
			port=ARGF();
			break;
		case 't':
			timeout = atoi(ARGF()) * 1000;
			break;
		case 'v':
			version = atoi(ARGF());
			break;
		case 'r':
			retries = atoi(ARGF());
			break;
		case 'c':
			rocomm = ARGF();
			break;
		case 'w':
			rwcomm = ARGF();
			break;
		case 'o':
			oids = ARGF();
			break;
		case 'V':
			values = ARGF();
			break;
	} ARGEND;

	if (host == nil)
		usage(cmd);

	sess = mksession(version, host, port, rocomm, rwcomm, timeout, retries);

	if (sess ==nil) {
		print("Error creating snmp session.\n");
		exits("Error creating session\n");
		return 1;
	} else {
		val = dosnmp(sess, oids, values, op, n, m);
		
		if (val != nil) {
			print("result:\n%s\n", val);
			free(val);
		}
		freesession(sess);
		return 0;		
	}
}
