#include <u.h>
#include <libc.h>
#include <ip.h>
#include "../lib/asn1.h"
#include "../lib/snmp.h"
#include "../lib/snmp_debug.h"


void usage(char *cmd)
{
		print("Usage: %s [-D]-h <host> [-p port] [-v ver] [-r <retries>] -c <community> -o <oid>\n", cmd);
		print("Example: %s -D -h 192.168.1.1 -p snmp -v 1 -r 2 -c public -o 1.3.6.1.2.1.1.1.0\n", cmd);
		print("\n");
		print("Options:\n");
		print("\t-D:\t\t\t\tUse it for debug\n");
		print("\t-h <host>:\t\tSet the destination host (no default)\n");
		print("\t-p <port>:\t\tSet the destination port (default = snmp)\n");
		print("\t-v <version>:\t\tSet the SNMP version [0 for V1, 1 for 2C] (default = 1)\n");
		print("\t-r <retries>:\t\tSet the retries if something is wrong (default = 3)\n");
		print("\t-c <community>:\tSet the community (no default)\n");
		print("\t-o <OID>:\t\tSet the OID [Object ID]\n");
		exits("Error in args");

}


int
main(int argc, char *argv[])
{
	char *cmd, *host, *port, *comm, *val, *oids;
	int version, retries, timeout;
	snmp_session *sess;

	/* default values */
	timeout = 5000;
	version = retries = 0;
	port = oids = comm = host = nil;

	cmd = argv[0];

	ARGBEGIN {
		case 'D':
			setdebug();
			break;
		case 'h':
			host=ARGF();
			break;
		case 'p':
			port=ARGF();
			break;
		case 't':
			timeout = atoi(ARGF()) * 1000;
			break;
		case 'v':
			version = atoi(ARGF());
			break;
		case 'r':
			retries = atoi(ARGF());
			break;
		case 'c':
			comm = ARGF();
			break;
		case 'o':
			oids = ARGF();
			break;
	} ARGEND;

	if (host == nil)
		usage(cmd);

	sess = mksession(version, host, port, comm, nil, timeout, retries);

	if (sess ==nil) {
		print("Error creating snmp session.\n");
		exits("Error creating session\n");
		return 1;
	} else {
		val = doWalk(sess, oids);
		
		if (val != nil) {
			print("result:\n%s\n", val);
			free(val);
		}
		freesession(sess);
		return 0;		
	}
}
