#include <u.h>
#include <libc.h>
#include <auth.h>
#include <fcall.h>
#include <thread.h>
#include <9p.h>
#include <bio.h>
#include <ndb.h>
#include "../lib/asn1.h"
#include "../lib/snmp.h"
#include "../lib/snmp_debug.h"
#include "../lib/fs.h"

void
fsread(Req *r)
{
	snmp_session *s;

	s = r->fid->file->aux;

	if (s == nil) {
		error(DEBUG, GOBACK, "fsread()", "Error creating the session.");
	} else {
		error(DEBUG, GOBACK, "fsread()", "Session created.");
	}

	/* Executing the doGet */
	print("%s\n", doGet(s, r->fid->file->name));

	respond(r, nil);
}

void
fswrite(Req *r)
{
	respond(r, Enowrite);
}

void
fscreate(Req *r)
{
	/* The filesystem do not support new files */
	respond(r, Enocreate);
}

void
fsopen(Req *r)
{
	respond(r, nil);
}

void
fsdestroyfile(File *f)
{

}

Srv fs = {
	.open	=	fsopen,
	.read		=	fsread,
	.write	=	fswrite,
	.create	=	fscreate,
};

void
usage(void)
{
	fprint(2, "usage: snmpfs [-D] -m <mtpt> -a <ip>\n");
	exits("usage");
}

int
createtree(char *address)
{
	Ndb *myndb;
	Ndbtuple *snmpinfo;
	Ndbtuple *tmpndbtuple;

	/* array with the information to create the snmp_session
	  * ["int version",		"char *host",	"char *port",	"char *rocommunity",
	  *  "char *rwcommunity",	"int timeout",	"int retries"]
	  */
	char *tmpchar;
	char *snmpparm[8];
	char version[8];
	char host[256];
	char port[10];
	char rocommunity[10];
	char rwcommunity[10];
	char timeout[12];
	char retries[12];
	char snmpoid[10000]; /* kix!!!!!! use the session->maxoid */
	snmp_fsinfo *fsinfo;

	snprint(host, 4, "%s", "sys");
	snprint(version, 8, "%s", "snmpver");
	snprint(port, 9, "%s", "snmpport");
	snprint(rocommunity, 10, "%s", "snmprocom");
	snprint(rwcommunity, 10, "%s", "snmprwcom");
	snprint(timeout, 12, "%s", "snmptimeout");
	snprint(retries, 12, "%s", "snmpretries");
	snprint(snmpoid, 10000, "%s", "snmpoid"); /* kix!!!!!! use the session->maxoid */

	snmpparm[0] = host;
	snmpparm[1] = version;
	snmpparm[2] = port;
	snmpparm[3] = rocommunity;
	snmpparm[4] = rwcommunity;
	snmpparm[5] = timeout;
	snmpparm[6] = retries;
	snmpparm[7] = snmpoid;

	myndb = malloc(sizeof(myndb)); /* Meter en el chatty */
	if(myndb == nil) {
		error(DEBUG, GOBACK, "createtree()", "Not enougth memory to read the ndb file.\n");
		/* kix, you must set here an exit(-1) */
	}

	myndb = ndbopen(NDBFILE);
	if (myndb != nil) {
		if (chatty9p)
			error(DEBUG, EQUAL, "createtree()", "%s file opened.", NDBFILE);

		/* Time to search the information */
		snmpinfo = ndbipinfo(myndb, "ip", address, snmpparm, 8);

		ndbclose(myndb);
		if (chatty9p)
			error(DEBUG, EQUAL, "createtree()", "%s file closed.", NDBFILE);
	} else {
		if (chatty9p)
			error(DEBUG, EQUAL, "createtree()", "Error opening the %s file.", NDBFILE);

		return(1);
	}

	/* We need to fill the information for the session */
	tmpndbtuple = snmpinfo;
	while(tmpndbtuple != nil) {
		if (chatty9p)
			error(DEBUG, EQUAL, "createtree()", "tuple: %s = %s", tmpndbtuple->attr, tmpndbtuple->val);

		/* Sys */
		if(!strcmp("sys", tmpndbtuple->attr)) {
			snprint(host, 255, "%s", tmpndbtuple->val);
			if (chatty9p)
				error(DEBUG, EQUAL, "createtree()", "Tuple assigned: host = %s", tmpndbtuple->val);
		}

		/* Version */
		if(!strcmp("snmpver", tmpndbtuple->attr)) {
			snprint(version, 7, "%s", tmpndbtuple->val);
			if (chatty9p)
				error(DEBUG, EQUAL, "createtree()", "Tuple assigned: version = %s", tmpndbtuple->val);
		}

		/* Port */
		if(!strcmp("snmpport", tmpndbtuple->attr)) {
			snprint(port, 8, "%s", tmpndbtuple->val);
			if (chatty9p)
				error(DEBUG, EQUAL, "createtree()", "Tuple assigned: port = %s", tmpndbtuple->val);
		}

		/* RO Community */
		if(!strcmp("snmprocom", tmpndbtuple->attr)) {
			snprint(rocommunity, 9, "%s", tmpndbtuple->val);
			if (chatty9p)
				error(DEBUG, EQUAL, "createtree()", "Tuple assigned: rocomm = %s", tmpndbtuple->val);
		}

		/* RW Community */
		if(!strcmp("snmprwcom", tmpndbtuple->attr)) {
			snprint(rwcommunity, 9, "%s", tmpndbtuple->val);
			if (chatty9p)
				error(DEBUG, EQUAL, "createtree()", "Tuple assigned: rwcomm = %s", tmpndbtuple->val);
		}

		/* Timeout */
		if(!strcmp("snmptimeout", tmpndbtuple->attr)) {
			snprint(timeout, 11, "%s", tmpndbtuple->val);
			if (chatty9p)
				error(DEBUG, EQUAL, "createtree()", "Tuple assigned: timeout = %s", tmpndbtuple->val);
		}

		/* Retries */
		if(!strcmp("snmpretries", tmpndbtuple->attr)) {
			snprint(port, 11, "%s", tmpndbtuple->val);
			if (chatty9p)
				error(DEBUG, EQUAL, "createtree()", "Tuple assigned: retries = %s", tmpndbtuple->val);
		}

		/* OID */
		if(!strcmp("snmpoid", tmpndbtuple->attr)) {
			snprint(snmpoid, 9999, "%s", tmpndbtuple->val);	/* kix */
			if (chatty9p)
				error(DEBUG, EQUAL, "createtree()", "Tuple assigned: OID = %s", tmpndbtuple->val);
		}

		/* The next */
		tmpndbtuple = tmpndbtuple->entry;
	}

	/* Now we have the values saved, creating the SNMP session ... */
	fsinfo = malloc(sizeof(struct snmp_fsinfo));

	fsinfo->snmpsess = mksession(atoi(version), host, port, rocommunity,
					rwcommunity, atoi(timeout), atoi(retries));
	if(fsinfo->snmpsess != nil) {
		if (chatty9p)
			error(DEBUG, EQUAL, "createtree()", "SNMP session created.");
	} else {
		error(FATAL, EQUAL, "createtree()", "Error creating the SNMP session\n");
	}

	/* QUEDA PREPARAR LOS OIDs Y LIMPIAR MEMORIA */
	/* Falta crear el borrado de ficheros */
	/* Los OIDs están preparados para usar "doGet(session, char *oids); */
	tmpchar = strtok(snmpoid, ",");
	while(tmpchar != nil) {
		File *f;
		char *user;

		user = getuser();

		error(DEBUG, EQUAL, "createtree()", "OID promoted to file: %s", tmpchar);
		f = createfile(fs.tree->root, tmpchar, user, 0400, fsinfo->snmpsess);
		tmpchar = strtok(nil, ",");
	}

	return(0);
}

void
main(int argc, char **argv)
{
	char *mtpt = nil;
	char *address = nil;

	fs.tree = alloctree(nil, nil, DMDIR|0777, fsdestroyfile);

	ARGBEGIN{
	case 'D':
		chatty9p++;
		break;
	case 'm':
		mtpt = EARGF(usage());
		break;
	case 'a':
		address = EARGF(usage());
		break;
	default:
		usage();
	}ARGEND;

	if (argc)
		usage();

	if (chatty9p) {
		/* Set the debug for snmp */
		setdebug();
		error(DEBUG, PLUS, "main()", "snmpfs.nopipe %d mtpt %s\n", fs.nopipe, mtpt);
	}

	if(mtpt == nil && address == nil)
		sysfatal("must specify -m and -a options");

	/* Mount the server */
	postmountsrv(&fs, nil, mtpt, MREPL|MCREATE);
	/* Create the tree */
	createtree(address);
	exits(0);
}
