#include <u.h>
#include <libc.h>
#include <bio.h>
#include <auth.h>
#include <fcall.h>
#include <thread.h>
#include <9p.h>
#include "../lib/asn1.h"
#include "../lib/snmp_asn1.h"
#include "../lib/snmp_debug.h"
#include "../lib/snmp.h"
/* Optional: #include "beer.h" or #include "caffeine.h" */

int alarmflag;
int readflag;

/* creates a new snmp request id */
/* based on rsc code */
int
mkreqid(void) 
{
	error(DEBUG, GOBACK, "mkreqid()", "Creating a Request ID.");
	return (nsec() & 0x7FFE) + 1;
	
}


/* routines to manage tuples */
int
linktuples(Tuple *father, Tuple *child)
{
	Tuple *auxtpl;

	error(DEBUG, PLUS, "linktuples()", "Linking tuples.");

	auxtpl = father;

	while(auxtpl->nexttuple != nil) {
		auxtpl = auxtpl->nexttuple;
	}

	auxtpl->nexttuple = child;

	error(DEBUG, MINUS, "linktuples()", "Linked.");

	return 0;
}

int
linkpkt(Packet *father, Packet *child)
{
	Packet *auxpkt;

	error(DEBUG, PLUS, "linkpkt()", "Linking packets.");

	auxpkt = father;

	while(auxpkt->nextpkt != nil) {
		auxpkt = auxpkt->nextpkt;
	}

	auxpkt->nextpkt = child;

	error(DEBUG, MINUS, "linkpkt()", "Linked.");

	return 0;
}

Tuple *
mktuple(Elem oid, Elem value)
{
	Tuple *aux;

	error(DEBUG, PLUS, "mktuple()", "Creating a Tuple.");

	/* Gabi, here I find a leak */
	aux = (Tuple*) malloc(sizeof(Tuple));
	aux->oid = oid;
	aux->value = value;
	aux->nexttuple = nil;

	error(DEBUG, MINUS, "mktuple()", "Created. addr = %p", aux);

	return aux;
}


/* kix! memory leak here */
void
freetuple(Tuple *t)
{
	error(DEBUG, PLUS, "freetuple()", "Freeing the memory of a Tuple.");

	if (t->nexttuple != nil) {
		/* go to the next node */
		freetuple(t->nexttuple);
	}

	/* Now, free the node */
	error(DEBUG, EQUAL, "freetuple()", "Freeing the Tuple. addr = %p", t);
	free(t);

	error(DEBUG, MINUS, "freetuple()", "Freeded.");
}

void
freepdu(Pdu *p)
{
	error(DEBUG, PLUS, "freepdu()", "Freeing the memory of a PDU.");

	freetuple(p->tuple);
	error(DEBUG, EQUAL, "freepdu()", "Freeing the PDU. addr = %p", p);
	free(p);

	error(DEBUG, MINUS, "freepdu()", "Freeded.");
}

void
freepkt(Packet *p)
{
	error(DEBUG, PLUS, "freepkt()", "Freeing the memory of a Packet.");

	if (p->nextpkt != nil) {
		/* go to the next node */
		freepkt(p->nextpkt);
	}

	/* Now, with the packet, first the PDU */
	freepdu(p->pdu);
	if (p->comm != nil) {
		error(DEBUG, EQUAL, "freepkt()", "Freeing the Packet-> community.");
//		free(p->comm);
	}
	error(DEBUG, EQUAL, "freepkt()", "Freeing the Packet. addr = %p", p);
	free(p);

	error(DEBUG, MINUS, "freepkt()", "Freeded.");
}

void
mkpdu(Packet *pkt)
{
	pkt->pdu = (Pdu*) malloc(sizeof(Pdu));
	pkt->nextpkt = nil;
	error(DEBUG, GOBACK, "mkpdu()", "Memory for the PDU allocated. addr = %p", pkt->pdu);
}

/* if community is nil, we only make room for the packet */
/* kix, revise for memory leak and cut this function */
Packet*
mkpkt(snmp_session *sess, int type, char *oids, char *values, int nonrep, int maxrep) 
{
	Packet  *pkt;
	Tuple *tmptpl;
	int i, noid, nval = 0;
	char **loids, **lvals = nil;
	Ints *myInts;

	error(DEBUG, PLUS, "mkpkt()", "Creating a Packet.");

	/* Get memory to create the packet */
	pkt = (Packet*) malloc(sizeof(Packet));
	/* Create the PDU */
	mkpdu(pkt);

	error(DEBUG, EQUAL, "mkpkt()", "Memory for the packet allocated. addr = %p", pkt);

	if (sess != nil) {
		error(DEBUG, EQUAL, "mkpkt()", "The Packet contains data.");

		if ((type == GetBulkRequest) && (sess->version < SNMP_VERSION_2) || (oids == nil)) {
			free(pkt);
			return(nil);
		} else {
			/* We must save the values of the oids to not modify it in the getfields */
			error(DEBUG, EQUAL, "mkpkt()", "Preparing the Request.");
			loids = (char**) malloc(sizeof(char*) * sess->maxoids);
			noid = getfields(strdup(oids), loids, sess->maxoids, 0, ",");
		}

		if (type == SetRequest) {
			if (values == nil) {
				free(pkt);
				return(nil);
			} else {
				/* We must save the values to not modify it in the getfields */
				error(DEBUG, EQUAL, "mkpkt()", "Preparing the SetRequest.");
				lvals = (char**) malloc(sizeof(char*) * sess->maxoids);
				nval = getfields(strdup(values), lvals, sess->maxoids, 0, ",");
			}
		}

		if ((nval != noid) && (type == SetRequest)) {
			error(DEBUG, EQUAL, "mkpkt()", "In a SetRequest the OIDs and Values must be equals.");
			free(pkt);
			return(nil);
		}

		/* Set the values in the packet */
		pkt->pdu->type = type;
		pkt->pdu->reqid = mkreqid();
		pkt->pdu->errstat = 0;
		pkt->pdu->erridx = 0;
		pkt->pdu->tlen = noid;
		pkt->pdu->nrep = nonrep;
		pkt->pdu->maxrep = maxrep;

		/* Prepare the oids */
		/* Must be at least one oid for the get or two for the set */
		myInts = snmp_mkoid(loids[0]);
		if (pkt->pdu->type == SetRequest) {
			pkt->pdu->tuple = mktuple(mkoid(myInts), char2asn(lvals[0]));
		} else {
			pkt->pdu->tuple = mktuple(mkoid(myInts), Null());
		}
		free(myInts);

		/* The next oids */
		for(i = 1; i < noid; i++) {
			myInts = snmp_mkoid(loids[i]);
			if (pkt->pdu->type == SetRequest) {
				tmptpl = mktuple(mkoid(myInts), char2asn(lvals[i]));		
			} else {
				tmptpl = mktuple(mkoid(myInts), Null());
			}
			linktuples(pkt->pdu->tuple, tmptpl);
			free(myInts);
		}
	}

/*	free(saved_oid); */
/*	free(saved_val); */

	error(DEBUG, MINUS, "mkpkt()", "Created.");

	return(pkt);
}


/* ASN.1 snmp packet encoding
  * 
  * message = SEQUECE
  *		version INTEGER
  *		community OCTECT_STRING
  *		PDU = IMPLICIT SEQUENCE
  */
int
encpkt(snmp_session *sess, Packet *pkt, Bytes **pbytes)
{
	Elem snmp;
	Elist * pdu;
	int err;
	Bytes *bsnmp;
	
	error(DEBUG, PLUS, "encpkt()", "Encapsulating the Packet.");

	if ((pkt == nil) || (pkt->pdu == nil)) {
		return 1;
	}

	switch(pkt->pdu->type) {
		case GetRequest: 
			error(DEBUG, EQUAL, "encpkt()", "Making a SNMP GetRequest.");
			pdu = snmprequest(pkt);
			break;
		case GetNextRequest:
			error(DEBUG, EQUAL, "encpkt()", "Making a SNMP NextRequest.");
			pdu = snmprequest(pkt);
			break;
		case GetBulkRequest:
			error(DEBUG, EQUAL, "encpkt()", "Making a SNMP BulkRequest.");
			pdu = snmpbulk(pkt);
			break;
		case SetRequest:
			error(DEBUG, EQUAL, "encpkt()", "Making a SNMP SetRequest.");
			pdu = snmprequest(pkt);
			break;
		default:
			return 1;
			break;
	}

	if ( pdu == nil ) {
		error(DEBUG, MINUS, "encpkt()", "The PDU is a null pointer.");
		return 1;
	} else {
		error(DEBUG, EQUAL, "encpkt()", "The PDU is not a null pointer.");
		snmp = snmpsecuence(sess, pdu);
		err = encode(snmp, &bsnmp);
		*pbytes = bsnmp;
	}

	/* Now, free the list */
	error(DEBUG, EQUAL, "encpkt()", "Free the old encapsulation PDU.");
	freeelist(pdu);

	error(DEBUG, MINUS, "encpkt()", "Packet encapsulation Success.");
	return(err);
}

/*
  * Var-Bind-List =SEQUENCE 		
  *	Var-Bind = SEQUENCE		
  *		oid	OBJECT IDENTIFIER	
  *		oid	OBJECT IDENTIFIER
  *		NULL
  */
Elem
mkvarbind(Tuple *tuple)
{
	Elist *x, *y = nil;
	Tuple *tmptpl, *storetpl, *newtpl, *revtpl;

	tmptpl = tuple;

	/* The mkel() function store the elements in reverse order
	  * therefore, I create a temporal list in reverse order too.
	  * One element is needed to make the header
	  */
	revtpl = mktuple(tmptpl->oid, tmptpl->value);
	tmptpl = tmptpl->nexttuple;

	/* When the header is created, continue with the next elements */
	while(tmptpl != nil) {
		newtpl = mktuple(tmptpl->oid, tmptpl->value);
		tmptpl = tmptpl->nexttuple;

		/* Link the tuple to the header, relink the header and go! */
		linktuples(newtpl, revtpl);
		revtpl = newtpl;
	}

	/* Here we have two lists, tuple (original), storetpl (reverse) */
	storetpl = revtpl;

	while(storetpl != nil) {
		x = mkel(Null(), nil); 			/* Gabi, memory leak here */
		x = mkel(storetpl->oid, x);
		y = mkel(mkseq(x), y); 		/* Gabi, memory leak here */

		storetpl = storetpl->nexttuple;
	}

	/* Now, destroy the temporal (reverse) list */
	freetuple(revtpl);

//	freetuple(tmptpl);
//	freetuple(storetpl);
//	freetuple(newtpl);
//	freetuple(revtpl);

	return mkseq(y);
}


Elist *
snmpbulk(Packet *pkt) {
	return(mkrequest(
		pkt->pdu->type,
		pkt->pdu->reqid,
		pkt->pdu->nrep,
		pkt->pdu->maxrep,
		mkvarbind(pkt->pdu->tuple)));
}


Elist *
snmprequest(Packet *pkt) {
	return(mkrequest(
		pkt->pdu->type,
		pkt->pdu->reqid,
		pkt->pdu->errstat,
		pkt->pdu->erridx,
		mkvarbind(pkt->pdu->tuple)));
}

Elem
snmpsecuence(struct snmp_session *sess, Elist *pdu) {
		/* kix, only with the rocommunity. You must set the rwcommunity */
		return(mkseq(mkel(mkint(sess->version),
			mkel(
			mkoctet((uchar*)sess->rocommunity, sess->rocommlen), pdu))));
}

int
decpdu(Elist *el, Packet *pkt)
{
	Elem *elem, last;
	Elist *pel = nil, *elvar = nil, *elbind = nil;
	Pdu *pdu;
	int err, tabsize = 0, i;
	Tuple *storetpl = nil, *tmptpl;

	pdu = pkt->pdu;

	error(DEBUG, PLUS, "dec_pdu()", "Decodifing a PDU.");

	elem = &el->hd;
	err = ASN_OK;

	/* Decode the reqid */
	if( is_int(elem, &pdu->reqid) ) {
		pel = el->tl;
		elem = &pel->hd;
	} else {
		error(DEBUG, MINUS, "dec_pdu()", "Reqid not found.");
		err=ASN_EINVAL;
	}

	/* Decode the errstat */
	if ( is_int(elem, &pdu->errstat) && (err == ASN_OK) ) {
		pel = pel->tl;
		elem = &pel->hd;
	} else {
		error(DEBUG, MINUS, "dec_pdu()", "Errstat not found.");
		err= ASN_EINVAL;
	}

	/* Decode the errindex */
	if ( is_int(elem, &pdu->erridx) && (err == ASN_OK) ) {
		pel = pel->tl;
		elem = &pel->hd;
	} else {
		error(DEBUG, MINUS, "dec_pdu()", "Erridx not found.");
		err = ASN_EINVAL;
	}

	/* As debbuging info, print the values */
	if (err == ASN_OK) {
		error(DEBUG, EQUAL, "dec_pdu()", "reqid = %d errstat = %d erridx = %d",
			pdu->reqid, pdu->errstat, pdu->erridx);
	}

	/* Decode the Varbind and get the number of elements */
	if ( (is_seq(elem, &elvar)) && (err == ASN_OK) ) {
		elem = &elvar->hd;
		tabsize = elistlen(elvar);
		error(DEBUG, EQUAL, "dec_pdu()", "vlen = %d", tabsize);
	} else {
		error(DEBUG, MINUS, "dec_pdu()", "Error decoding var list.");
		err = ASN_EINVAL;
	}

	/* Decode each oid */
	if (err == ASN_OK) {
		for(i = 0; i < tabsize; i++) {
			if ( is_seq(elem, &elbind) ) {
				pel = elbind;
				last = pel->hd;
				pel = pel->tl;
				if ( last.val.tag == VObjId ) {
					/* Se añade la PDU */
					if (i == 0) {
						/* First oid */
						/* Must be one oid */
						storetpl = mktuple(last, pel->hd);
					} else {
						/* The next oids */
						tmptpl = mktuple(last, pel->hd);
						linktuples(storetpl, tmptpl);
					}
				} else {
					error(DEBUG, MINUS, "dec_pdu()", "Error decoding var list.");
					return ASN_EINVAL;
				}
			} else {
				error(DEBUG, MINUS, "dec_pdu()", "Error decoding bind list.");
				err = ASN_EINVAL;
			}
			/* Prepare to the next */
			elvar = elvar->tl;
			elem = &elvar->hd;
		}
		pdu->tuple = storetpl;
	}

	if (err == ASN_OK) {
		pkt->pdu = pdu;
		setmalloctag(pdu, getcallerpc(el));
		freeelist(elbind);
		freeelist(elvar);
		error(DEBUG, MINUS, "dec_pdu()", "PDU Decodification Success.");
	}
	
	return err;
}


int
decpkt(uchar *buff, int n, Packet *pkt) 
{
	Bytes *bbuff, *bcomm;
	Elem emess, *ever, *ecomm, *pdu, *epdu;
	Elist *elmess, *el, *elpdu;

	error(DEBUG, PLUS, "decpkt()", "Decodifing the Packet.");
	
	bbuff = makebytes(buff, n);

	if (decode(bbuff->data, bbuff->len, &emess) != ASN_OK) {
		error(DEBUG, MINUS, "decpkt()", "Error in ASN.1 decode.");
		return -1;
	}

	freebytes(bbuff);

	if( !is_seq(&emess, &elmess)  ) {
		error(DEBUG, MINUS, "decpkt()", "Cannot recognize message.");
		return -2;
	}
	if ( elistlen(elmess) != 3 ) {
		error(DEBUG, MINUS, "decpkt()", "Cannot recognize message: n = %d", elistlen(elmess));
		return -3;
	}

	ever = &elmess->hd;
	el = elmess->tl;
	ecomm = &el->hd;
	el = el->tl;
	epdu = &el->hd;

	if ( !is_int(ever, &pkt->version) ) {
		error(DEBUG, MINUS, "decpkt()", "Error decoding version field.");
		return -4;
	}

	if ( !is_octetstring(ecomm, &bcomm) ) {
		error(DEBUG, MINUS, "decpkt()", "Error decoding community field.");
		return -5;
	}

	/* We make here the memory allocation for the comm beacuse we
	  * do not known the length of the community field */
	pkt->comm = (char *) malloc(bcomm->len + 1);
	snprint(pkt->comm, bcomm->len, "%s", (char *)bcomm->data);
	pkt->comm[bcomm->len]='\0';

	error(DEBUG, EQUAL, "decpkt()", "Version = %d Community = %s", pkt->version, pkt->comm);
	
	pdu = pdu2asn(epdu);

	if ( pdu == nil ) {
		error(DEBUG, MINUS, "decpkt()", "Error decoding pdu: %r");
		return -6;
	}

	if ( !is_seq(pdu, &elpdu) ) {
		error(DEBUG, MINUS, "decpkt()", "Error decoding pdu.");
		return -7;
	}

	if ( decpdu(elpdu, pkt) != ASN_OK ) {
		error(DEBUG, MINUS, "decpkt()", "Error decoding pdu field.");
		return -8;
	}

	freeelist(elmess);

	error(DEBUG, MINUS, "decpkt()", "Packet decodification Success.");

	return 0;
}

/* timeout
  * Code from rsc */
int
alarmtr(void *v, char *why)
{

	error(DEBUG, PLUS, "alarmtr()", "Alarm captured.");

	if(!readflag) {
		error(DEBUG, MINUS, "alarmtr()", "Alarm served (exit 0).");
		return 0;
	}

	if(strcmp(why, "alarm") == 0) {
		alarmflag++;
		error(DEBUG, MINUS, "alarmtr()", "Alarm served (exit 1).");
		return 1;
	}

	error(DEBUG, MINUS, "alarmtr()", "Alarm served (exit 0).");
	return 0;
}

int
talk(snmp_session *sess, Packet *snd, Packet *rcv)
{
	int n, retries, reqid, fd;
	uchar *rcvbuf;
	Bytes *snmp;

	error(DEBUG, PLUS, "talk()", "Entering in the talk.");

	rcvbuf = (uchar*) malloc(sess->rcvMsgMaxSize);

	if (rcvbuf == nil) {
		error(DEBUG, MINUS, "talk()", "No memory available for the reception buffer.");
		return -5;
	}

	if (encpkt(sess, snd, &snmp)) {
		error(DEBUG, MINUS, "talk()", "Error encoding Packet to ASN1.");
		free(rcvbuf);
		return -4;
	}

	reqid = snd->pdu->reqid;
	retries = sess->retries;

	/* And ... the little file descriptor */
	fd = dial(netmkaddr(sess->host, "udp", sess->port), 0, 0, 0);
	
	do {
		n = write(fd, snmp->data, snmp->len);
		
		if (n != snmp->len) {
			error(DEBUG, MINUS, "talk()", "Error writing to %d", fd);
			free(rcvbuf);
			return -3;
		}

		/* set alarm to exit the while when timeout reached */
		alarmflag = 0;
		readflag = 1;
		atnotify(alarmtr, 1);
		alarm(sess->timeout);
		error(DEBUG, EQUAL, "talk()", "timeout = %d", sess->timeout);
		do {
			n = read(fd, rcvbuf, sess->rcvMsgMaxSize);
			if (n > 0)	
				if (decpkt(rcvbuf, n, rcv))
					continue;
				else if ( reqid == rcv->pdu->reqid ) {
					freebytes(snmp);
					error(DEBUG, MINUS, "talk()", "pkt readed id = %d", reqid);
					free(rcvbuf);
					close(fd);
					alarm(0);
					return (0);
				}
		} while(!alarmflag);

		if (alarmflag)
				retries--;
		
	} while (retries);
	
	close(fd);
	free(rcvbuf);
	freebytes(snmp);
	alarm(0);
	return -1;
}
