#include "../lib/snmp_session.h"

/* error codes for pdu parsing */
enum {
	noError,
	tooBig,
	noSuchName,
	badValue,
	readOnly,
	genErr,
	noAccess,
	wrongType,
	wrongLength,
	wrongEncoding,
	wrongValue,
	noCreation,
	inconsistenValue,
	resourceUnavailable,
	commitFailed,
	undoFailed,
	authorizationError,
	notWritable,
	inconsistentName
};

/* error codes for trap-pdu */
enum {
	coldstart,
	warmstart,
	linkdown,
	linkup,
	authfailure,
	neightborloss,
	enterprise
};

/* pdu types for snmp v1 and v2 (implicit sequences) */
enum {
	GetRequest 		= 0xa0,
	GetNextRequest 	= 0xa1,
	GetResponse 		= 0xa2,
	SetRequest 		= 0xa3,
	Trap 			= 0xa4,
	GetBulkRequest 	= 0xa5,
	InformRequest 		= 0xa6,
	SNMPv2Trap 		= 0xa7
};

/* snmp structures */
typedef struct Pdu Pdu;
typedef struct Packet Packet;
typedef struct Traps Traps;
typedef struct Tuple Tuple;

struct Pdu {
	uchar type;	/* for all PDUs */
	int reqid;		/* for all PDUs */
	int errstat;		/* for all PDUs */
	int erridx;		/* for all PDUs */
	int nrep;		/* getbulk */
	int maxrep;	/* getbulk */
	int tlen;		/* getbulk */
	Tuple *tuple;
};

struct Packet {
	int version;
	char *comm;
	Pdu *pdu;
	Packet *nextpkt;
};

struct Traps {
	char *oid;
	char *addr;
	int type;
	char *ts;
	Tuple *tuple;
};

struct Tuple {
	Elem oid;
	Elem value;
	Tuple *nexttuple;
};

int alarmtr(void *v,char *why);
int comp(Packet * p1, Packet * p2); 
int encpkt(snmp_session *sess, Packet *p, Bytes **pbytes);
int decpdu(Elist *el, Packet *pkt);
int decpkt(uchar *buff, int n, Packet *ppkt);
int mkreqid(void);
int talk(snmp_session *s, Packet *snd, Packet *rcv);
int linkpkt(Packet *father, Packet *child);

Packet * mkreq(snmp_session *s, int type, char **oids, char **values, int len);
Packet * mkpkt(snmp_session *sess, int type, char *oids, char *values, int nonrep, int maxrep);
void mkpdu(Packet *pkt);

void freepdu(Pdu *p);
void freepkt(Packet * p);

Elem mkvarbind(Tuple *tuple);

Elist * mkgetnreq(Packet *p);
Elist * mkgetbulkreq(Packet *p);
Elist * mkgetbulkreq(Packet *p);
Elist * mkgetnreq(Packet *p);
Elist * snmprequest(Packet *pkt);
Elist * snmpbulk(Packet *pkt);
Tuple * mktuple(Elem oid, Elem value);
int linktuples(Tuple *father, Tuple *child);
Elem snmpsecuence(struct snmp_session *sess, Elist *pdu);