#include <u.h>
#include <libc.h>
#include <bio.h>
#include <auth.h>
#include <fcall.h>
#include <thread.h>
#include <9p.h>
#include "../lib/snmp_debug.h"

/* debug or error */
int
error(int level, int inc, char *func, char *format, ... )
{
	va_list args;
	char lasterror[ERRMAX];

	va_start(args, format);
	vsnprint(lasterror,ERRMAX, format, args);
	va_end(args);
	werrstr(lasterror);

	if (level) {
		/* Increment the function level, if needed */
		switch(inc) {
			case PLUS:
			case GOBACK:
				SPACES++;
				break;
		}

		/* Print the first part of the debug */


		switch(level) {
			case 1:
				fprint(2, "[snmpfs] Debug: [");
				break;
			case 2:
				fprint(2, "[snmpfs] FATAL: [");
				break;
		}

		/* Print the level of function */
		if (SPACES > 0) {
			for(int i=0; i<SPACES; i++) {
				fprint(2, "=");
			}
		}
		/* And print the last part */
		fprint(2, "] %s: %r\n", func);

		/* Check if is a fatal error and exits */
		if (level == 2) {
			abort();
		}

		/* Decrement the function level, if needed */
		switch(inc) {
			case GOBACK:
			case MINUS:
				SPACES--;
				break;
		}
	}

	return 0;
}

void
setdebug(void)
{
	FATAL = 0;
	DEBUG = 0;
	SPACES = 0;

	DEBUG++;
}