#include <u.h>
#include <libc.h>
#include <bio.h>
#include <auth.h>
#include <fcall.h>
#include <thread.h>
#include <9p.h>
#include "../lib/asn1.h"
#include "../lib/snmp_debug.h"
#include "../lib/snmp_session.h"


snmp_session *
mksession(int version, char *host, char *port, char *rocommunity, char *rwcommunity, int tmout, int retrs)
{
	snmp_session *sess;

	error(DEBUG, PLUS, "mkssesion()", "Creating a SNMP session.");

	/* Version */
	if (( version < 0 ) || ( version > 3 )) {
		version = SNMP_VERSION;
	}

	/* Destination Host */
	if (host == nil) {
		return nil;
	}

	/* Destination Port */
	if (port == nil)
		port = smprint("%s", SNMP_PORT);

	/* RO Community */
	if (rocommunity == nil)
		rocommunity = smprint("%s", SNMP_ROCOMMUNITY);

	/* RW Community */
	if (rwcommunity == nil)
		rwcommunity = smprint("%s", SNMP_RWCOMMUNITY);

	/* Timeout */
	if (tmout == 0)
		tmout = SNMP_TIMEOUT;

	/* Retries */
	if (retrs == 0)
		retrs = SNMP_RETRIES;

	/* memory!! */
	sess = emalloc9p(sizeof(snmp_session));

	/* Value's Asignation */
	sess->version = version;
	sess->host = strdup(host);
	sess->port = strdup(port);
	sess->rocommunity = strdup(rocommunity);
	sess->rwcommunity = strdup(rwcommunity);
	sess->rocommlen = strlen(sess->rocommunity);
	sess->rwcommlen = strlen(sess->rwcommunity);
	sess->timeout = tmout;
	sess->retries = retrs;

	/* Values sets in the code, kix change it to parameter */
	sess->sndMsgMaxSize = SNMP_MAX_SND_PKT;
	sess->rcvMsgMaxSize = SNMP_MAX_RCV_PKT;
	sess->maxoids = SNMP_MAX_OIDS;
	
	error(DEBUG, MINUS, "mkssesion()", "Session created.");

	return sess;
}


void
freesession(snmp_session *sess)
{
	error(DEBUG, PLUS, "freesession()", "Freeing the memory of Session.");

	free(sess->host);
	free(sess->port);
	free(sess->rocommunity);
	free(sess->rwcommunity);
	free(sess);

	error(DEBUG, MINUS, "freesession()", "Memory freed.");
}
